C***************************************************************************
C!   sspline.f
C***************************************************************************

      subroutine sspline_d(lon, lat, y, n, lambda, varhat, gcv, coefc, 
     $     coefd, yhat, Q, ldq, S, tau, work, lwork, d, e)

C
C     Purpose:
C     ========
C
C        Fortran interface for solving spherical spline problem.
C
C     Arguments:
C     ==========
C
C        lon: input, double array, dimension n by 1, the longitudes.
C        lat: input, double array, dimension n by 1, the latitudes.
C          y: input, double array, dimension n by 1, the observations.
C          n: input, integer, the dimension of lon, lat and y.
C
C     lambda: output, double, the optimal lambda chosen by gcv.
C     varhat: output, double, the estimated variance.
C        gcv: output, double, the optimal gcv value.
C      coefc: output, double array, dimension n by 1, the coefficients 
C             in the estimated function.
C      coefd: output, double, the coefficients in the estimated
C             function.
C       yhat: ouput/workspace, double array, the predicted values at
C             the observed points.
C          Q: workspace, double array, dimension ldq by n, used for
C             storing with the kernel matrix Q.
C        ldq: input, integer, leading dimension of the working arrry Q.
C          S: workspace, double array, dimension n by 1.
C        tau: workspace, double array, dimension n by 1, used in the matrix
C             decomposition.
C       work: workspace, double array, dimension lwork by 1, used in the
C             matrix decomposition.
C       d, e: workspace, double array, dimension n by 1, used in the matrix
C             decomposition.
C
C     ***Note***: all the longitudes and latitudes are in the radiant measure.
C


C**********************************************************************
C
C     constant for debugging
C
      parameter (MAXD=50)

C**********************************************************************

C
C     declaration of the arguments
C
      integer n, ldq, lwork
      double precision lon(n), lat(n), y(n)
      double precision yhat(n), S(n), Q(ldq, n)
      double precision tau(n), work(lwork)
      double precision d(n), e(n), coefc(n)
      double precision coefd, lambda, varhat, gcv

C
C     declaration of the local variables
C
      double precision  low, up, prec

C**********************************************************************
C
C     variables for debugging
C
      double precision obs(MAXD), cumsum(MAXD), origk(MAXD, MAXD)
      double precision inprod(MAXD)
      integer ldk

      ldk = MAXD

C**********************************************************************


C
C     generate the matrix Q from the longitudes and the latitudes.
C
      call dmatgen(lon, lat, n, Q)


C**********************************************************************
C
C     copy the orignal matrix Q into a backup matrix origk
C
      do 20, j = 1, n
         do 10, i = 1, n
            origk(i, j) = Q(i, j)
 10      continue
 20   continue

C
C     copy the original observation into a backup vector obs
C
      call dcopy(n, y, 1, obs, 1)

C**********************************************************************

C
C     save the y vector into yhat for later use
C
      call dcopy(n, y, 1, yhat, 1)

C
C     set up the problem: find the orthogonal matrix, eigenvalues and etc. 
C
      call dgcvst(Q, ldq, n, y, S, d, e, tau, work, lwork)

      print *, 'y ='
      print *, y

C
C     set the range of searching for lambda
C
      prec = 1.0d0
      do 30 while (1.0d0 + prec > 1.0d0)
         prec = prec/2.0d0
 30   continue
      
      low = 2.0d0 * prec
      up  = 1.0d0 / low 
C
C     search for the lambda
C
      call gcvsch(y(2), d, n, low, up, lambda, gcv, varhat)

C
C     solve the equation using the lambda found in the above step.
C
      call deqsv(Q, ldq, n, y, d, lambda, coefc, coefd, S, 
     $           work, lwork, e)


C********************************************************************
C
C     calculate the matrix sum: K + n*lambda*I
C
      do 40, i = 1, n
         origk(i, i) = origk(i, i) + n*lambda
 40   continue

C
C     calculate the matrix-vector product (K + n*lambda*I)*c
C
      call dgemv('N', n, n, 1.0d0, origk, ldk, coefc, 1, 0.0d0,
     $     cumsum, 1)

C
C     restore the vector S to all 1
C
      do 50, i = 1, n
         S(i) = 1.0d0
 50   continue

C
C     calculate the sum of the above vector and S*d (d is a scalar)
C
      call daxpy(n, coefd, S, 1, cumsum, 1)

C
C     print the debugging result
C
      print *
      print *, 'the calculated cumsum is'
      print *, cumsum

      print *
      print *, 'the original obs was'
      print *, obs
      
      print *
      print *, 'the S vector now is'
      print *, S
      
      print *
      print *, 'the dot product of S and c is'
      print *, ddot(n, S, 1, coefc, 1)  

C*******************************************************************

C
C     calculate the estimated y value: yhat = y - n*lambda*coefc
C
      call daxpy(n, -dble(n)*lambda, coefc, 1, yhat, 1)
 
      return
      end















